"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = __importDefault(require("node-fetch"));
const enums_1 = require("./enums");
const querystring = __importStar(require("querystring"));
const config_1 = require("./config");
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const base_1 = require("./base");
const iltorb_1 = require("iltorb");
const errors_1 = require("./errors");
const cookie_version_matcher_1 = require("./cookie-version-matcher");
const logger = base_1.container.get(base_1.TYPES.Logger);
const httpClient = base_1.container.get(base_1.TYPES.Client);
class Fragment {
    constructor(config) {
        this.name = config.name;
    }
}
exports.Fragment = Fragment;
class FragmentBFF extends Fragment {
    constructor(config) {
        super({ name: config.name });
        this.handler = {};
        this.config = config;
        if (this.config.versionMatcher) {
            this.versionMatcher = new cookie_version_matcher_1.CookieVersionMatcher(this.config.versionMatcher);
        }
        this.prepareHandlers();
    }
    /**
     * Renders fragment: data -> content
     * @param {object} req
     * @param {string} version
     * @returns {Promise<HandlerDataResponse>}
     */
    render(req, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const handler = this.handler[version] || this.handler[this.config.version];
            const clearedRequest = this.clearRequest(req);
            if (handler) {
                if (handler.data) {
                    let dataResponse;
                    try {
                        dataResponse = yield handler.data(clearedRequest);
                    }
                    catch (e) {
                        logger.error(`Failed to fetch data for fragment ${this.config.name}`, req.url, req.query, req.params, req.headers, e);
                        return {
                            $status: 500
                        };
                    }
                    if (dataResponse.data) {
                        const renderedPartials = handler.content(clearedRequest, dataResponse.data);
                        delete dataResponse.data;
                        return Object.assign({}, renderedPartials, dataResponse);
                    }
                    else {
                        return dataResponse;
                    }
                }
                else {
                    throw new Error(`Failed to find data handler for fragment. Fragment: ${this.config.name}, Version: ${version || this.config.version}`);
                }
            }
            else {
                throw new Error(`Failed to find fragment version. Fragment: ${this.config.name}, Version: ${version || this.config.version}`);
            }
        });
    }
    /**
     * Renders placeholder
     * @param {object} req
     * @param {string} version
     * @returns {string}
     */
    placeholder(req, version) {
        const fragmentVersion = (version && this.config.versions[version]) ? version : this.config.version;
        const handler = this.handler[fragmentVersion];
        if (handler) {
            return handler.placeholder();
        }
        else {
            throw new Error(`Failed to find fragment version. Fragment: ${this.config.name}, Version: ${version || this.config.version}`);
        }
    }
    /**
     * Renders error
     * @param {object} req
     * @param {string} version
     * @returns {string}
     */
    errorPage(req, version) {
        const fragmentVersion = (version && this.config.versions[version]) ? version : this.config.version;
        const handler = this.handler[fragmentVersion];
        if (handler) {
            return handler.error();
        }
        else {
            throw new Error(`Failed to find fragment version. Fragment: ${this.config.name}, Version: ${version || this.config.version}`);
        }
    }
    /**
     * Purifies req.path, req.query from Puzzle elements.
     * @param req
     * @returns {*}
     */
    clearRequest(req) {
        const clearedReq = Object.assign({}, req);
        if (req.query) {
            delete clearedReq.query[config_1.RENDER_MODE_QUERY_NAME];
            delete clearedReq.query[config_1.PREVIEW_PARTIAL_QUERY_NAME];
            delete clearedReq.query[config_1.DEBUG_QUERY_NAME];
        }
        if (req.path) {
            clearedReq.path = req.path.replace(`/${this.name}`, '');
        }
        return clearedReq;
    }
    /**
     * Check module type
     */
    checkModuleType(fragmentModule) {
        if (typeof fragmentModule === "function")
            return fragmentModule(base_1.container);
        return fragmentModule;
    }
    /**
     * Resolve handlers based on configuration
     */
    prepareHandlers() {
        Object.keys(this.config.versions).forEach(version => {
            const configurationHandler = this.config.versions[version].handler;
            if (configurationHandler) {
                this.handler[version] = configurationHandler;
            }
            else {
                const module = require(path_1.default.join(process.cwd(), `/src/fragments/`, this.config.name, version));
                this.handler[version] = this.checkModuleType(module);
            }
        });
    }
}
exports.FragmentBFF = FragmentBFF;
class FragmentStorefront extends Fragment {
    constructor(name, from) {
        super({ name });
        this.primary = false;
        this.shouldWait = false;
        this.from = from;
    }
    /**
     * Updates fragment configuration
     * @param {IExposeFragment} config
     * @param {string} gatewayUrl
     * @param gatewayName
     * @param {string | undefined} assetUrl
     */
    update(config, gatewayUrl, gatewayName, assetUrl) {
        if (assetUrl) {
            this.assetUrl = url_1.default.resolve(assetUrl, this.name);
        }
        this.fragmentUrl = url_1.default.resolve(gatewayUrl, this.name);
        const hostname = url_1.default.parse(gatewayUrl).hostname;
        if (hostname) {
            this.gatewayPath = hostname;
        }
        this.gatewayName = gatewayName;
        this.config = config;
        if (this.config && this.config.versionMatcher) {
            this.versionMatcher = new cookie_version_matcher_1.CookieVersionMatcher(this.config.versionMatcher);
        }
        if (this.config && this.config.render.error && !this.cachedErrorPage) {
            this.getErrorPage();
        }
    }
    detectVersion(cookie, preCompile = false) {
        if (!this.config)
            return '0';
        const cookieKey = this.config.testCookie;
        const cookieVersion = cookie[cookieKey];
        if (cookieVersion) {
            return cookieVersion;
        }
        if (!preCompile && this.versionMatcher) {
            const version = this.versionMatcher.match(cookie);
            if (version)
                return version;
        }
        return this.config.version;
    }
    /**
     * Returns fragment placeholder as promise, fetches from gateway
     * @returns {Promise<string>}
     */
    getPlaceholder() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`Trying to get placeholder of fragment: ${this.name}`);
            if (!this.config) {
                logger.error(new Error(`No config provided for fragment: ${this.name}`));
                return '';
            }
            if (!this.config.render.placeholder) {
                logger.error(new Error('Placeholder is not enabled for fragment'));
                return '';
            }
            return node_fetch_1.default(`${this.fragmentUrl}/placeholder`, {
                headers: {
                    gateway: this.gatewayName
                }
            })
                .then(res => res.text())
                .then(html => {
                logger.info(`Received placeholder contents of fragment: ${this.name}`);
                return html;
            })
                .catch(err => {
                logger.error(`Failed to fetch placeholder for fragment: ${this.fragmentUrl}/placeholder`, err);
                return '';
            });
        });
    }
    /**
     * Returns fragment error as promise, fetches from gateway
     * @returns { Promise<string> }
     */
    getErrorPage() {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`Trying to get error page of fragment: ${this.name}`);
            if (!this.config || !this.config.render.error) {
                logger.warn(new Error('Error is not enabled for fragment'));
                return '';
            }
            if (this.cachedErrorPage) {
                return this.cachedErrorPage;
            }
            return node_fetch_1.default(`${this.fragmentUrl}/error`, {
                headers: {
                    gateway: this.gatewayName
                }
            })
                .then(res => res.json())
                .then(html => {
                this.cachedErrorPage = html;
                return html;
            })
                .catch(err => {
                logger.error(`Failed to fetch error for fragment: ${this.fragmentUrl}/error`, err);
                return '';
            });
        });
    }
    /**
     * Fetches fragment content as promise, fetches from gateway
     * Returns {
     *  html: {
     *    Partials
     *  },
     *  status: gateway status response code
     * }
     * @param attribs
     * @param req
     * @returns {Promise<IFragmentContentResponse>}
     */
    // @nrSegmentAsync("fragment.getContent", true)
    getContent(attribs = {}, req) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`Trying to get contents of fragment: ${this.name}`);
            if (!this.config) {
                logger.error(new Error(`No config provided for fragment: ${this.name}`));
                return {
                    status: 500,
                    html: {},
                    headers: {},
                    cookies: {},
                    model: {}
                };
            }
            let query = Object.assign({}, attribs, { __renderMode: enums_1.FRAGMENT_RENDER_MODES.STREAM });
            let parsedRequest;
            const requestConfiguration = {
                timeout: this.config.render.timeout || config_1.DEFAULT_CONTENT_TIMEOUT,
            };
            if (req) {
                if (req.url) {
                    parsedRequest = url_1.default.parse(req.url);
                    query = Object.assign({}, query, req.query);
                }
                if (req.headers) {
                    requestConfiguration.headers = req.headers;
                    requestConfiguration.headers['originalurl'] = req.url;
                    requestConfiguration.headers['originalpath'] = req.path;
                }
            }
            requestConfiguration.headers = Object.assign({}, requestConfiguration.headers, { gateway: this.gatewayName }) || { gateway: this.gatewayName };
            delete query.from;
            delete query.name;
            delete query.partial;
            delete query.primary;
            delete query.shouldwait;
            const routeRequest = req && parsedRequest ? `${parsedRequest.pathname.replace('/' + this.name, '')}?${querystring.stringify(query)}` : `/?${querystring.stringify(query)}`;
            return httpClient.get(`${this.fragmentUrl}${routeRequest}`, this.name, Object.assign({ json: true, gzip: true }, requestConfiguration)).then(res => {
                logger.info(`Received fragment contents of ${this.name} with status code ${res.response.statusCode}`);
                return {
                    status: res.data.$status || res.response.statusCode,
                    headers: res.data.$headers || {},
                    cookies: res.data.$cookies || {},
                    html: res.data,
                    model: res.data.$model || {}
                };
            }).catch((err) => __awaiter(this, void 0, void 0, function* () {
                logger.error(new errors_1.PuzzleError(errors_1.ERROR_CODES.FAILED_TO_GET_FRAGMENT_CONTENT, this.name, `${this.fragmentUrl}${routeRequest}`), this.name, `${this.fragmentUrl}${routeRequest}`, `${this.fragmentUrl}${routeRequest}`, Object.assign({ json: true }, requestConfiguration), err);
                const errorPage = yield this.getErrorPage();
                return {
                    status: errorPage ? 200 : 500,
                    html: errorPage ? errorPage : {},
                    headers: {},
                    cookies: {},
                    model: {}
                };
            }));
        });
    }
    /**
     * Returns asset content
     * @param {string} name
     * @param targetVersion
     * @returns {Promise<string>}
     */
    getAsset(name, targetVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`Trying to get asset: ${name}`);
            if (!this.config) {
                logger.error(new Error(`No config provided for fragment: ${this.name}`));
                return null;
            }
            let fragmentVersion = this.config;
            if (targetVersion !== this.config.version && this.config.passiveVersions && this.config.passiveVersions[targetVersion]) {
                fragmentVersion = this.config.passiveVersions[targetVersion];
            }
            const asset = fragmentVersion.assets.find(asset => asset.name === name);
            if (!asset) {
                logger.error(new Error(`Asset not declared in fragments asset list: ${name}`));
                return null;
            }
            const link = (asset.link || `${this.fragmentUrl}/static/${asset.fileName}`) + `?__version=${targetVersion}`;
            return node_fetch_1.default(link, {
                headers: {
                    gateway: this.gatewayName
                }
            }).then((res) => __awaiter(this, void 0, void 0, function* () {
                logger.info(`Asset received: ${name}`);
                const encoding = res.headers.get('content-encoding');
                switch (encoding) {
                    case enums_1.CONTENT_ENCODING_TYPES.BROTLI:
                        return yield iltorb_1.decompress(yield res.buffer());
                    default:
                        return yield res.text();
                }
            })).catch(e => {
                logger.error(new Error(`Failed to fetch asset from gateway: ${this.fragmentUrl}/static/${asset.fileName}`));
                return null;
            });
        });
    }
    /**
     * Returns asset path
     * @param {string} name
     * @returns {string}
     */
    getAssetPath(name) {
        if (!this.config) {
            logger.error(new Error(`No config provided for fragment: ${this.name}`));
            return null;
        }
        const asset = this.config.assets.find(asset => asset.name === name);
        if (!asset) {
            logger.error(new Error(`Asset not declared in fragments asset list: ${name}`));
            return null;
        }
        return asset.link || `${this.assetUrl || this.fragmentUrl}/static/${asset.fileName}`;
    }
}
exports.FragmentStorefront = FragmentStorefront;
//# sourceMappingURL=fragment.js.map