"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const enums_1 = require("./enums");
const node_fetch_1 = __importDefault(require("node-fetch"));
const config_1 = require("./config");
const base_1 = require("./base");
const util_1 = require("util");
const puzzle_warden_1 = __importDefault(require("puzzle-warden"));
const logger = base_1.container.get(base_1.TYPES.Logger);
const httpClient = base_1.container.get(base_1.TYPES.Client);
class GatewayStorefrontInstance {
    constructor(gatewayConfig, authToken, gatewayUpdateInterval) {
        this.events = new events_1.EventEmitter();
        this.satisfyAmount = config_1.SATISFY_COMPILE_AMOUNT;
        this.intervalId = null;
        this.pendingUpdate = {
            hash: null,
            count: 0
        };
        this.name = gatewayConfig.name;
        this.url = gatewayConfig.url;
        this.authToken = authToken;
        this.satisfyAmount = gatewayUpdateInterval || config_1.SATISFY_COMPILE_AMOUNT;
        httpClient.init('PuzzleJs Storefront');
        this.assetUrl = gatewayConfig.assetUrl;
        this.fetch();
    }
    /**
     * Starts updating gateway by polling with the provided miliseconds
     * @param {number} pollingInterval
     */
    startUpdating(pollingInterval = config_1.DEFAULT_POLLING_INTERVAL) {
        this.intervalId = setInterval(this.fetch.bind(this), pollingInterval);
    }
    /**
     * Stops polling
     */
    stopUpdating() {
        if (this.intervalId) {
            clearInterval(this.intervalId);
        }
    }
    /**
     * Fetches gateway condifuration and calls this.bind
     */
    fetch() {
        return __awaiter(this, void 0, void 0, function* () {
            const headers = {
                gateway: this.name,
            };
            if (this.authToken) {
                headers["x-authorization"] = this.authToken;
            }
            try {
                const res = yield node_fetch_1.default(this.url, {
                    headers
                });
                const json = yield res.json();
                this.update(json);
            }
            catch (e) {
                logger.error(`Failed to fetch gateway configuration: ${this.name}`, e);
            }
        });
    }
    /**
     * Updates gateway configuration and if hash changed emits GATEWAY_UPDATED event
     * @param {IExposeConfig} data
     */
    update(data) {
        if (!this.config) {
            logger.info(`Gateway is ready: ${this.name}`);
            this.connectWarden(data);
            this.config = data;
            this.events.emit(enums_1.EVENTS.GATEWAY_READY, this);
        }
        else {
            if (data.hash !== this.config.hash) {
                if (this.satisfyUpdate(data.hash)) {
                    logger.info(`Gateway is updated: ${this.name}`);
                    this.connectWarden(data);
                    this.config = data;
                    this.events.emit(enums_1.EVENTS.GATEWAY_UPDATED, this);
                }
            }
            else {
                this.resetUpdateStatus(data.hash);
            }
        }
    }
    satisfyUpdate(hash) {
        if (this.pendingUpdate.hash === hash) {
            this.pendingUpdate.count++;
            if (this.pendingUpdate.count <= this.satisfyAmount) {
                this.resetUpdateStatus(hash);
                return true;
            }
        }
        else {
            this.resetUpdateStatus(hash);
        }
        return false;
    }
    connectWarden(data) {
        for (const key in data.fragments) {
            const fragment = data.fragments[key];
            if (fragment.warden && fragment.warden.identifier) {
                if (this.shouldUpdateWarden(key, fragment.warden)) {
                    puzzle_warden_1.default.register(key, fragment.warden);
                }
            }
            else {
                puzzle_warden_1.default.unregisterRoute(key);
            }
        }
    }
    resetUpdateStatus(hash) {
        this.pendingUpdate.hash = hash;
        this.pendingUpdate.count = 0;
    }
    shouldUpdateWarden(fragmentName, newConfiguration) {
        if (!this.config || !this.config.fragments[fragmentName])
            return true;
        return !util_1.isDeepStrictEqual(this.config.fragments[fragmentName].warden, newConfiguration);
    }
}
exports.GatewayStorefrontInstance = GatewayStorefrontInstance;
//# sourceMappingURL=gatewayStorefront.js.map