"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const template_1 = require("./template");
const enums_1 = require("./enums");
const config_1 = require("./config");
class Page {
    constructor(html, gatewayMap, name) {
        this.name = name;
        this.ready = false;
        this.responseHandlers = {};
        this.waitingCompilers = {};
        this.prgEnabled = false;
        this.rawHtml = html;
        this.template = new template_1.Template(html, this.name);
        this.gatewayDependencies = this.template.getDependencies();
        this.preparePageDependencies(gatewayMap);
        this.checkPageReady();
    }
    /**
     * Request handler, compiles template if not exists
     * @param {{cookies: ICookieObject}} req
     * @param {object} res
     * @returns {Promise<void>}
     */
    handle(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const isDebug = config_1.DEBUG_INFORMATION || (req.query && req.query.hasOwnProperty(config_1.DEBUG_QUERY_NAME));
            const handlerVersion = this.getHandlerVersion(req.cookies);
            const compileVersion = `${handlerVersion}_${isDebug}`;
            if (!this.responseHandlers[compileVersion]) {
                const waitedCompileProcess = this.waitingCompilers[compileVersion];
                if (waitedCompileProcess) {
                    yield waitedCompileProcess;
                }
                else {
                    this.waitingCompilers[compileVersion] = this.template.compile(req.cookies, isDebug);
                    yield this.waitingCompilers[compileVersion]
                        .then(handler => {
                        this.responseHandlers[compileVersion] = handler;
                    });
                }
            }
            this.responseHandlers[compileVersion](req, res);
        });
    }
    post(req, res, next) {
        if (this.prgEnabled) {
            res.redirect(303, req.originalUrl);
        }
        else {
            next();
        }
    }
    reCompile() {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultVersion = this.getHandlerVersion({}, true);
            this.responseHandlers[`${defaultVersion}_true`] = yield this.template.compile({}, true, true);
            this.responseHandlers[`${defaultVersion}_false`] = yield this.template.compile({}, false, true);
        });
    }
    /**
     * Prepares dependencies and subscribes to events.
     * @param {IGatewayMap} gatewayMap
     */
    preparePageDependencies(gatewayMap) {
        Object.keys(gatewayMap)
            .filter(gatewayName => this.gatewayDependencies.gateways[gatewayMap[gatewayName].name])
            .forEach(gatewayName => {
            gatewayMap[gatewayName].events.on(enums_1.EVENTS.GATEWAY_UPDATED, this.gatewayUpdated.bind(this));
            this.gatewayDependencies.gateways[gatewayName].gateway = gatewayMap[gatewayName];
            if (!!gatewayMap[gatewayName].config) {
                this.gatewayDependencies.gateways[gatewayName].ready = true;
            }
            else {
                gatewayMap[gatewayName].events.once(enums_1.EVENTS.GATEWAY_READY, this.gatewayReady.bind(this));
            }
        });
    }
    /**
     * Checks for page dependencies are ready
     */
    checkPageReady() {
        if (Object.keys(this.gatewayDependencies.gateways).filter(gatewayName => !this.gatewayDependencies.gateways[gatewayName].ready).length === 0) {
            this.ready = true;
        }
    }
    /**
     * Based on test cookies returns handler key
     * @returns {string}
     * @param query
     * @param cookies
     */
    getHandlerVersion(cookies, preCompile = false) {
        return Object.values(this.gatewayDependencies.fragments)
            .reduce((key, fragment) => {
            return `${key}_${fragment.instance.name}|${fragment.instance.detectVersion(cookies, preCompile)}`;
        }, '');
    }
    updatePrgStatus() {
        this.prgEnabled = Object
            .values(this.gatewayDependencies.fragments)
            .some(fragment => !!(fragment.instance.config && fragment.instance.config.prg && fragment.instance.primary));
    }
    /**
     * Called on GATEWAY_UPDATED
     * @param {GatewayStorefrontInstance} gateway
     */
    gatewayUpdated(gateway) {
        this.updateFragmentsConfig(gateway);
        this.template.load();
        this.updatePrgStatus();
        this.reCompile();
    }
    /**
     * Updates fragment config based on gateway configuration
     * @param {GatewayStorefrontInstance} gateway
     */
    updateFragmentsConfig(gateway) {
        Object.values(this.gatewayDependencies.fragments).forEach(fragment => {
            if (fragment.gateway === gateway.name && gateway.config) {
                fragment.instance.update(gateway.config.fragments[fragment.instance.name], gateway.url, gateway.name, gateway.assetUrl);
            }
        });
    }
    /**
     * Called on GATEWAY_READY
     * @param {GatewayStorefrontInstance} gateway
     */
    gatewayReady(gateway) {
        this.gatewayDependencies.gateways[gateway.name].ready = true;
        this.updateFragmentsConfig(gateway);
        this.updatePrgStatus();
        this.checkPageReady();
    }
}
exports.Page = Page;
//# sourceMappingURL=page.js.map