"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const enums_1 = require("./lib/enums");
const resourceFactory_1 = __importDefault(require("./resourceFactory"));
const enums_2 = require("./enums");
const clean_css_1 = __importDefault(require("clean-css"));
class ResourceInjector {
    constructor(fragments, pageName, cookies) {
        this.fragments = fragments;
        this.cookies = cookies;
        this.fragmentFingerPrints = [];
        this.assets = [];
        this.dependencies = [];
        this.pageName = pageName;
        this.prepare();
    }
    /**
     * Injects prepared assets to dom
     * @param { CheerioStatic } dom
     * @returns {Promise<void>}
     */
    injectAssets(dom) {
        this.assets.forEach(asset => {
            if (asset.loadMethod === enums_1.RESOURCE_LOADING_TYPE.ON_RENDER_START) {
                asset.preLoaded = true;
                if (Array.isArray(asset.dependent) && asset.dependent.length > 0) {
                    this.dependencies.forEach(dependency => {
                        if (Array.isArray(asset.dependent) && asset.dependent.indexOf(dependency.name) > -1 && !dependency.preLoaded) {
                            dependency.preLoaded = true;
                            ResourceInjector.injectDefaultJsAsset(dependency, dom);
                        }
                    });
                }
                ResourceInjector.injectDefaultJsAsset(asset, dom);
            }
        });
    }
    /**
     * Injects library config to dom
     * @param { CheerioStatic } dom
     * @returns {Promise<void>}
     */
    injectLibraryConfig(dom) {
        dom('head').prepend(ResourceInjector.wrapJsAsset({
            content: `{puzzleLibContent}PuzzleJs.emit('${enums_1.EVENT.ON_RENDER_START}');PuzzleJs.emit('${enums_1.EVENT.ON_CONFIG}','${JSON.stringify(this.libraryConfig)}');`,
            injectType: enums_2.RESOURCE_INJECT_TYPE.INLINE,
            name: 'puzzle-lib',
            link: '',
            executeType: enums_2.RESOURCE_JS_EXECUTE_TYPE.SYNC
        }));
    }
    /**
     * Merges, minifies stylesheets and inject them to dom
     * @param { CheerioStatic } dom
     * @param { boolean } precompile
     * @returns {Promise<void>}
     */
    injectStyleSheets(dom, precompile) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const _CleanCss = new clean_css_1.default({
                    level: {
                        1: {
                            all: true
                        }
                    }
                });
                const cssData = {
                    styleSheets: [],
                    dependencyNames: []
                };
                for (const fragment of Object.values(this.fragments)) {
                    yield this.loadCSSData(cssData, fragment, precompile);
                }
                if (cssData.styleSheets.length > 0) {
                    const output = _CleanCss.minify(cssData.styleSheets.join(''));
                    const addEscapeCharacters = output.styles.replace(/content:"/g, 'content:"\\');
                    dom('head').append(`<style puzzle-dependency="dynamic-css" dependency-list="${cssData.dependencyNames.join(',')}">${addEscapeCharacters}</style>`);
                }
                resolve();
            }));
        });
    }
    /**
     * Wraps js asset based on its configuration
     * @param {IWrappingJsAsset} asset
     * @returns {string}
     */
    static wrapJsAsset(asset) {
        if (asset.injectType === enums_2.RESOURCE_INJECT_TYPE.EXTERNAL && asset.link) {
            return `<script puzzle-dependency="${asset.name}" src="${asset.link}" type="text/javascript"${asset.executeType}> </script>`;
        }
        else if (asset.injectType === enums_2.RESOURCE_INJECT_TYPE.INLINE && asset.content) {
            return `<script puzzle-dependency="${asset.name}" type="text/javascript">${asset.content}</script>`;
        }
        else {
            //todo handle error
            return `<!-- Failed to inject asset: ${asset.name} -->`;
        }
    }
    /**
     * Load css data(stylesheets and dependency name list) and push to first parameter
     * @param { } cssData
     * @param {FragmentStorefront} fragment
     * @param {boolean} precompile
     * @returns {Promise<>}
     */
    loadCSSData(cssData, fragment, precompile) {
        return __awaiter(this, void 0, void 0, function* () {
            const targetVersion = fragment.detectVersion(this.cookies, precompile);
            const config = this.getFragmentConfig(fragment, targetVersion);
            if (config) {
                const cssAssets = config.assets.filter(asset => asset.type === enums_1.RESOURCE_TYPE.CSS);
                const cssDependencies = config.dependencies.filter(dependency => dependency.type === enums_1.RESOURCE_TYPE.CSS);
                for (const asset of cssAssets) {
                    const assetContent = yield fragment.getAsset(asset.name, targetVersion);
                    if (assetContent) {
                        cssData.styleSheets.push(assetContent.toString());
                        cssData.dependencyNames.push(asset.name);
                    }
                }
                for (const dependency of cssDependencies) {
                    if (!cssData.dependencyNames.includes(dependency.name)) {
                        cssData.dependencyNames.push(dependency.name);
                        cssData.styleSheets.push(yield resourceFactory_1.default.instance.getRawContent(dependency.name));
                    }
                }
            }
        });
    }
    ;
    /**
     * Returns fragment config using targeted version
     * @param { FragmentStorefront } fragment
     * @param { string } targetVersion
     * @returns { }
     */
    getFragmentConfig(fragment, targetVersion) {
        if (!fragment.config)
            return;
        if (fragment.config.version === targetVersion || !fragment.config.passiveVersions || !fragment.config.passiveVersions[targetVersion]) {
            return fragment.config;
        }
        return fragment.config.passiveVersions[targetVersion];
    }
    /**
     * Prepare assets, fragmentFingerPrints, dependencies and library config
     * @returns {Promise<void>}
     */
    prepare() {
        for (let fragment of Object.values(this.fragments)) {
            if (!fragment.config)
                continue;
            const targetVersion = fragment.detectVersion(this.cookies);
            const config = this.getFragmentConfig(fragment, targetVersion);
            this.prepareFragmentFingerPrint(fragment);
            this.prepareAssets(config);
            this.prepareDependencies(config);
        }
        this.libraryConfig = {
            page: this.pageName,
            fragments: this.fragmentFingerPrints,
            assets: this.assets.filter(asset => asset.type === enums_1.RESOURCE_TYPE.JS),
            dependencies: this.dependencies
        };
    }
    /**
     * Prepare injectable assets from fragment and push to this.assets
     * @param { } config
     */
    prepareAssets(config) {
        config.assets.forEach((asset) => {
            this.assets.push({
                loadMethod: typeof asset.loadMethod !== 'undefined' ? asset.loadMethod : enums_1.RESOURCE_LOADING_TYPE.ON_PAGE_RENDER,
                name: asset.name,
                dependent: asset.dependent || [],
                type: asset.type,
                link: asset.link,
                preLoaded: false
            });
        });
    }
    /**
     * Prepare injectable dependencies from fragment and push to this.dependencies
     * @param { } config
     * @returns {IPageFragmentConfig}
     */
    prepareDependencies(config) {
        config.dependencies.forEach(dependency => {
            const dependencyData = resourceFactory_1.default.instance.get(dependency.name);
            if (dependencyData && dependencyData.link && !this.dependencies.find(dependency => dependency.name === dependencyData.name)) {
                this.dependencies.push({
                    name: dependency.name,
                    link: dependencyData.link,
                    type: dependency.type,
                    preLoaded: false
                });
            }
        });
    }
    /**
     * Prepare fragment fingerprint from given fragment and push to this.fragmentFingerPrints
     * @param {FragmentStorefront} fragment
     */
    prepareFragmentFingerPrint(fragment) {
        this.fragmentFingerPrints.push({
            name: fragment.name,
            chunked: fragment.config ? (fragment.shouldWait || (fragment.config.render.static || false)) : false
        });
    }
    /**
     * Inject default JS assets
     * @param { } jsAsset
     * @param { CheerioStatic } dom
     * @returns {string}
     */
    static injectDefaultJsAsset(jsAsset, dom) {
        if (jsAsset.type === enums_1.RESOURCE_TYPE.JS) {
            dom('body').append(ResourceInjector.wrapJsAsset({
                content: ``,
                injectType: enums_2.RESOURCE_INJECT_TYPE.EXTERNAL,
                name: jsAsset.name,
                link: jsAsset.link,
                executeType: enums_2.RESOURCE_JS_EXECUTE_TYPE.SYNC
            }));
        }
    }
}
exports.default = ResourceInjector;
//# sourceMappingURL=resource-injector.js.map