"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = __importDefault(require("node-fetch"));
const enums_1 = require("./enums");
const util_1 = require("./util");
const enums_2 = require("./lib/enums");
const singletonSymbol = Symbol();
class ResourceFactory {
    constructor(enforcer) {
        this.resources = {};
        if (enforcer !== singletonSymbol)
            throw new Error("Cannot construct singleton, use .singleton");
    }
    static get instance() {
        if (!this.singleton) {
            this.singleton = new ResourceFactory(singletonSymbol);
        }
        return this.singleton;
    }
    /**
     * Registers new dependency
     * @param {IFileResourceStorefrontDependency} dependency
     */
    registerDependencies(dependency) {
        this.validateDependency(dependency);
        this.resources[dependency.name] = dependency;
    }
    /**
     * Returns dependency object
     * @param {string} dependencyName
     * @returns {IFileResourceStorefrontDependency | null}
     */
    get(dependencyName) {
        return this.resources[dependencyName] || null;
    }
    getRawContent(dependencyName) {
        return __awaiter(this, void 0, void 0, function* () {
            const dependency = this.resources[dependencyName];
            if (dependency && dependency.link) {
                const req = yield node_fetch_1.default(dependency.link);
                return yield req.text();
            }
            else if (dependency.content) {
                return dependency.content;
            }
            else {
                return `<!-- Puzzle dependency: ${dependencyName} not found -->`;
            }
        });
    }
    /**
     * Returns dependency content
     * @param {string} dependencyName
     * @returns {string}
     */
    getDependencyContent(dependencyName, injectType) {
        return __awaiter(this, void 0, void 0, function* () {
            const dependency = this.resources[dependencyName];
            if (!dependency) {
                return `<!-- Puzzle dependency: ${dependencyName} not found -->`;
            }
            if (injectType === enums_1.RESOURCE_INJECT_TYPE.INLINE) {
                if (dependency.type === enums_2.RESOURCE_TYPE.JS) {
                    if (dependency.content) {
                        return `<script puzzle-dependency="${dependency.name}" type="text/javascript">${dependency.content}</script>`;
                    }
                    else if (dependency.link) {
                        const req = yield node_fetch_1.default(dependency.link), content = yield req.text();
                        return `<script puzzle-dependency="${dependency.name}" type="text/javascript">${content}</script>`;
                    }
                }
                else if (dependency.type === enums_2.RESOURCE_TYPE.CSS) {
                    if (dependency.content) {
                        return `<style puzzle-dependency="${dependency.name}" type="text/css">${dependency.content}</style>`;
                    }
                    else if (dependency.link) {
                        const req = yield node_fetch_1.default(dependency.link), content = yield req.text();
                        return `<style puzzle-dependency="${dependency.name}" type="text/css">${content}</style>`;
                    }
                }
            }
            else if (injectType === enums_1.RESOURCE_INJECT_TYPE.EXTERNAL) {
                if (dependency.type === enums_2.RESOURCE_TYPE.JS) {
                    if (dependency.link) {
                        return `<script puzzle-dependency="${dependency.name}" src="${dependency.link}" type="text/javascript"> </script>`;
                    }
                    else if (dependency.content) {
                        const url = `/static/${dependency.name}.min.js`;
                        util_1.pubsub.emit(enums_1.EVENTS.ADD_ROUTE, {
                            path: url,
                            method: enums_1.HTTP_METHODS.GET,
                            handler(req, res) {
                                res.set('content-type', 'text/javascript');
                                res.send(dependency.content);
                            }
                        });
                        return `<script puzzle-dependency="${dependency.name}" src="${url}" type="text/javascript"> </script>`;
                    }
                }
                else if (dependency.type === enums_2.RESOURCE_TYPE.CSS) {
                    if (dependency.link) {
                        return `<link puzzle-dependency="${dependency.name}" rel="stylesheet" href="${dependency.link}" />`;
                    }
                    else if (dependency.content) {
                        const url = `/static/${dependency.name}.min.css`;
                        util_1.pubsub.emit(enums_1.EVENTS.ADD_ROUTE, {
                            path: url,
                            method: enums_1.HTTP_METHODS.GET,
                            handler(req, res) {
                                res.set('content-type', 'text/css');
                                res.send(dependency.content);
                            }
                        });
                        return `<link puzzle-dependency="${dependency.name}" rel="stylesheet" href="${url}" />`;
                    }
                }
            }
            else {
                if (dependency.type === enums_2.RESOURCE_TYPE.JS) {
                    if (dependency.link) {
                        return `<script puzzle-dependency="${dependency.name}" src="${dependency.link}" type="text/javascript"> </script>`;
                    }
                    else if (dependency.content) {
                        return `<script puzzle-dependency="${dependency.name}" type="text/javascript">${dependency.content}</script>`;
                    }
                }
                else if (dependency.type === enums_2.RESOURCE_TYPE.CSS) {
                    if (dependency.link) {
                        return `<link puzzle-dependency="${dependency.name}" rel="stylesheet" href="${dependency.link}" />`;
                    }
                    else if (dependency.content) {
                        return `<style puzzle-dependency="${dependency.name}" type="text/css">${dependency.content}</style>`;
                    }
                }
            }
            return `<!-- Puzzle dependency: ${dependencyName} failed to inject because there is no content or link property -->`;
        });
    }
    /**
     * Checks if dependency is valid for registration
     * @param {IFileResourceStorefrontDependency} dependency
     */
    validateDependency(dependency) {
        if (!dependency.content && !dependency.link) {
            throw new Error('Content or link is required for dependency');
        }
    }
}
ResourceFactory.singleton = false;
exports.default = ResourceFactory;
//# sourceMappingURL=resourceFactory.js.map