"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const helmet_1 = __importDefault(require("helmet"));
require("reflect-metadata");
const morgan_1 = __importDefault(require("morgan"));
const body_parser_1 = __importDefault(require("body-parser"));
const enums_1 = require("./enums");
const logger_1 = require("./logger");
const util_1 = require("./util");
const shrink_ray_current_1 = __importDefault(require("shrink-ray-current"));
const compression_1 = __importDefault(require("compression"));
const inversify_1 = require("inversify");
const response_time_1 = __importDefault(require("response-time"));
const config_1 = require("./config");
const spdy = __importStar(require("spdy"));
const https_1 = __importDefault(require("https"));
const morganLoggingLevels = [
    'Date: [:date[clf]]',
    'IP: :remote-addr',
    'Client-IP: :req[client-ip]',
    'REQ: :method :url',
    'RES: :status :response-time ms',
    'UA: :user-agent',
    'x-correlationId: :req[x-correlationId]',
    'x-agentname: :req[x-agentname]',
    'referer: :req[referer]',
];
const compressionFilter = {
    filter(req) {
        return !req.query[config_1.NO_COMPRESS_QUERY_NAME];
    }
};
let Server = class Server {
    constructor() {
        this.app = express_1.default();
        this.server = null;
        this.addMiddlewares();
        util_1.pubsub.on(enums_1.EVENTS.ADD_ROUTE, (e) => {
            this.addRoute(e.path, e.method, e.handler);
        });
    }
    /**
     * Sets spdy protocol configuration.
     * @param {ISpdyConfiguration} options
     */
    useProtocolOptions(options) {
        if (options) {
            this.spdyConfiguration = {
                cert: options.cert,
                key: options.key,
                passphrase: options.passphrase,
                spdy: {
                    "x-forwarded-for": true,
                    protocols: options.protocols,
                    connection: {
                        windowSize: 1024 * 1024,
                        autoSpdy31: false
                    }
                }
            };
        }
    }
    /**
     * Register new middleware to route
     * @param {string | null} path
     * @param {(req: Request, res: Response, next: NextFunction) => any} handler
     */
    addUse(path, handler) {
        if (path) {
            this.app.use(path, handler);
        }
        else {
            this.app.use(handler);
        }
    }
    /**
     * Registers static routes
     * @param {string | null} path
     * @param {string} source
     * @param {serveStatic.ServeStaticOptions} staticOptions
     */
    setStatic(path, source, staticOptions) {
        if (!staticOptions) {
            this.addUse(path, express_1.default.static(source));
        }
        else {
            this.addUse(path, express_1.default.static(source, staticOptions));
        }
    }
    /**
     * Adds new route
     * @param {string} path
     * @param {HTTP_METHODS} method
     * @param {(req: Request, res: Response, next: NextFunction) => any} handler
     * @param {RequestHandlerParams[]} middlewares
     */
    addRoute(path, method, handler, middlewares = []) {
        this.app[method](path, middlewares, handler);
    }
    /**
     * Starts server
     * @param {number} port
     * @param {Function} cb
     * @param ipv4
     */
    listen(port, cb, ipv4) {
        const args = [port];
        if (ipv4) {
            args.push('0.0.0.0');
        }
        args.push((e) => {
            cb && cb(e);
        });
        if (this.spdyConfiguration && (this.spdyConfiguration.spdy.protocols.includes(enums_1.TRANSFER_PROTOCOLS.H2) || this.spdyConfiguration.spdy.protocols.includes(enums_1.TRANSFER_PROTOCOLS.SPDY))) {
            this.server = spdy.createServer(this.spdyConfiguration, this.app);
            this.server.listen.apply(this.server, args);
        }
        else {
            if (this.spdyConfiguration && this.spdyConfiguration.cert && this.spdyConfiguration.key) {
                this.server = https_1.default.createServer({
                    cert: this.spdyConfiguration.cert,
                    key: this.spdyConfiguration.key
                }, this.app);
                this.server.listen.apply(this.server, args);
            }
            else {
                this.server = this.app.listen.apply(this.app, args);
            }
        }
        if (this.server) {
            this.server.timeout = config_1.GLOBAL_REQUEST_TIMEOUT;
        }
    }
    /**
     * Adds custom headers
     * @param {Array<ICustomHeader>} customHeaders
     */
    addCustomHeaders(customHeaders) {
        if (customHeaders) {
            this.addUse(null, (req, res, next) => {
                customHeaders.forEach((customHeader) => {
                    let value = customHeader.value.toString();
                    if (customHeader.isEnv && process.env[value]) {
                        value = process.env[value];
                    }
                    res.header(customHeader.key, value);
                });
                next();
            });
        }
    }
    /**
     * Clears instances, stops listening
     */
    close() {
        if (this.server) {
            this.server.close();
            this.server = null;
            this.app = express_1.default();
            this.addMiddlewares();
        }
    }
    /**
     * Predefined middlewares
     * @returns {boolean}
     */
    addMiddlewares() {
        this.app.use(response_time_1.default());
        if (config_1.USE_MORGAN)
            this.app.use(morgan_1.default(morganLoggingLevels.join('||'), { stream: logger_1.Logger.prototype }));
        if (config_1.USE_HELMET)
            this.app.use(helmet_1.default());
        this.app.use(body_parser_1.default.urlencoded({ extended: true }));
        this.app.use(body_parser_1.default.json());
        this.app.use(cookie_parser_1.default());
        const compressionMethod = config_1.BROTLI ? shrink_ray_current_1.default : compression_1.default;
        this.app.use(compressionMethod(compressionFilter));
    }
};
Server = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], Server);
exports.Server = Server;
//# sourceMappingURL=server.js.map