"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gatewayStorefront_1 = require("./gatewayStorefront");
const page_1 = require("./page");
const async_1 = __importDefault(require("async"));
const enums_1 = require("./enums");
const util_1 = require("./util");
const decorators_1 = require("./decorators");
const base_1 = require("./base");
const server_1 = require("./server");
const resourceFactory_1 = __importDefault(require("./resourceFactory"));
const config_1 = require("./config");
const configurator_1 = require("./configurator");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const logger = base_1.container.get(base_1.TYPES.Logger);
let Storefront = class Storefront {
    /**
     * Storefront Instance
     * @param {IStorefrontConfig} storefrontConfig
     * @param {Server} _server
     */
    constructor(storefrontConfig, _server) {
        this.pages = {};
        this.gateways = {};
        this.gatewaysReady = 0;
        this.server = _server || base_1.container.get(base_1.TYPES.Server);
        if (storefrontConfig instanceof configurator_1.StorefrontConfigurator) {
            this.config = storefrontConfig.configuration;
        }
        else {
            this.config = storefrontConfig;
        }
        this.bootstrap();
    }
    /**
     * Starts storefront instance
     * @param {Function} cb
     */
    init(cb) {
        logger.info('Starting Puzzle Storefront');
        async_1.default.series([
            this.registerDependencies.bind(this),
            this.waitForGateways.bind(this),
            this.registerDebugScripts.bind(this),
            this.addCustomHeaders.bind(this),
            this.addHealthCheckRoute.bind(this),
            this.preLoadPages.bind(this),
            this.addPageRoute.bind(this)
        ], err => {
            if (!err) {
                this.server.listen(this.config.port, () => {
                    logger.info(`Storefront is listening on port ${this.config.port}`);
                    cb && cb();
                }, this.config.ipv4);
            }
            else {
                throw err;
            }
        });
    }
    preLoadPages(cb) {
        Promise.all(Object.values(this.pages).map((page) => __awaiter(this, void 0, void 0, function* () {
            logger.info(`Preloading page: ${page.name}`);
            yield page.reCompile();
            logger.info(`Preloaded page: ${page.name}`);
        }))).then(() => cb());
    }
    bootstrap() {
        if (!fs_1.default.existsSync(config_1.TEMP_FOLDER)) {
            fs_1.default.mkdirSync(config_1.TEMP_FOLDER);
        }
        this.server.useProtocolOptions(this.config.spdy);
        this.createStorefrontPagesAndGateways();
    }
    /**
     * Creates static routes for debugging scripts
     * @param {Function} cb
     * @returns {Promise<void>}
     */
    registerDebugScripts(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            this.server.addRoute(config_1.PUZZLE_DEBUGGER_LINK, enums_1.HTTP_METHODS.GET, (req, res) => {
                res.sendFile(path_1.default.join(__dirname, './public/puzzle_debug.js'));
            });
            cb(null);
        });
    }
    /**
     * Waits for gateways to be prepared
     * @param {Function} cb
     * @returns {Promise<void>}
     */
    waitForGateways(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            while (Object.keys(this.gateways).length != this.gatewaysReady) {
                yield util_1.wait(config_1.GATEWAY_PREPERATION_CHECK_INTERVAL);
            }
            cb(null);
        });
    }
    /**
     * Creates gateway pages, pages and subscribes event to gateways to track ready status
     */
    createStorefrontPagesAndGateways() {
        this.config.gateways.forEach(gatewayConfiguration => {
            const gateway = new gatewayStorefront_1.GatewayStorefrontInstance(gatewayConfiguration, this.config.authToken, this.config.satisfyUpdateCount);
            gateway.events.once(enums_1.EVENTS.GATEWAY_READY, () => {
                this.gatewaysReady++;
            });
            gateway.startUpdating();
            this.gateways[gatewayConfiguration.name] = gateway;
        });
        this.config.pages.forEach(pageConfiguration => {
            this.pages[pageConfiguration.url.toString()] = new page_1.Page(pageConfiguration.html, this.gateways, pageConfiguration.name);
        });
    }
    /**
     * Registers provided dependencies in storefront configuration
     * @param {Function} cb
     */
    registerDependencies(cb) {
        this.config.dependencies.forEach(dependency => {
            logger.info(`Registering Dependency: ${dependency.name}`);
            resourceFactory_1.default.instance.registerDependencies(dependency);
        });
        cb();
    }
    /**
     * Adds healthcheck route.
     * @param {Function} cb
     */
    addHealthCheckRoute(cb) {
        logger.info(`Registering healthcheck routes: ${enums_1.HEALTHCHECK_PATHS}`);
        this.server.addRoute(enums_1.HEALTHCHECK_PATHS, enums_1.HTTP_METHODS.GET, (req, res) => {
            res.status(enums_1.HTTP_STATUS_CODE.OK).end();
        });
        cb();
    }
    /**
     * Adds custom headers
     * @param {Function} cb
     */
    addCustomHeaders(cb) {
        this.server.addCustomHeaders(this.config.customHeaders);
        cb();
    }
    /**
     * Adds page routes then connects with page instance responsible for it.
     * @param {Function} cb
     */
    addPageRoute(cb) {
        this.config.pages.forEach(page => {
            const targetPage = page.url.toString();
            logger.info(`Adding page ${page.name} route: ${targetPage}`);
            this.server.addRoute(page.url, enums_1.HTTP_METHODS.GET, (req, res, next) => {
                logger.info(`Request route name: ${page.name} - ${req.url} - ${JSON.stringify(req.headers)}`);
                if (typeof page.condition === 'function' ? page.condition(req) : true) {
                    this.pages[targetPage].handle(req, res);
                }
                else {
                    next();
                }
            });
            this.server.addRoute(page.url, enums_1.HTTP_METHODS.POST, (req, res, next) => {
                this.pages[targetPage].post(req, res, next);
            });
        });
        cb();
    }
};
__decorate([
    decorators_1.callableOnce,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function]),
    __metadata("design:returntype", void 0)
], Storefront.prototype, "init", null);
Storefront = __decorate([
    decorators_1.sealed,
    __metadata("design:paramtypes", [Object, server_1.Server])
], Storefront);
exports.Storefront = Storefront;
//# sourceMappingURL=storefront.js.map