import { FragmentStorefront } from "./fragment";
import { FragmentModel, ICookieMap, IFragmentEndpointHandler, IReplaceAsset, IReplaceSet } from "./types";
import { TemplateClass } from "./templateClass";
import express from "express";
interface CompressionStreamResponse extends express.Response {
    flush: () => void;
}
export declare class Template {
    rawHtml: string;
    private name?;
    dom: CheerioStatic;
    fragments: {
        [name: string]: FragmentStorefront;
    };
    pageClass: TemplateClass;
    constructor(rawHtml: string, name?: string | undefined);
    /**
     * Loads html template into Cheerio instance
     */
    load(): void;
    /**
     * Returns fragment dependencies
     * @returns {IPageDependentGateways}
     */
    getDependencies(): {
        gateways: {};
        fragments: {};
    };
    /**
     * Compiles template and returns a function that can handle the request.
     * @param {ICookieMap} testCookies
     * @param {boolean} isDebug
     * @returns {Promise<IFragmentEndpointHandler>}
     */
    compile(testCookies: ICookieMap, isDebug?: boolean, precompile?: boolean): Promise<IFragmentEndpointHandler>;
    /**
     * Bind user class to page
     */
    private bindPageClass;
    /**
     * Appends placeholders to reserved locations
     * @param {IChunkedReplacementSet[]} chunkedReplacements
     * @returns {Promise<void>}
     */
    private appendPlaceholders;
    /**
     * Replaces static fragments with their content on vDOM
     * @param {FragmentStorefront[]} fragments
     * @param {IReplaceAsset[]} replaceAssets
     * @returns {Promise<void>}
     */
    private replaceStaticFragments;
    private replaceEmptyTags;
    /**
     * Replaces waited fragments with their content on first flush string.
     * @param {IReplaceSet[]} waitedFragments
     * @param {string} template
     * @param req
     * @param isDebug
     * @returns {Promise<IWaitedResponseFirstFlush>}
     */
    private replaceWaitedFragments;
    /**
     * Maps fragment response variable model into PuzzleLib
     * @param {{name: string}} fragment
     * @param {FragmentModel} fragmentPageModel
     * @param {boolean} isDebug
     * @returns {string}
     */
    static fragmentModelScript(fragment: {
        name: string;
    }, fragmentPageModel: FragmentModel, isDebug?: boolean): string;
    /**
     * Handles non chunked page renders
     * @param firstFlushHandler
     * @param waitedFragments
     * @param isDebug
     * @param req
     * @param res
     */
    nonChunkedHandler(firstFlushHandler: Function, waitedFragments: IReplaceSet[], isDebug: boolean, req: express.Request, res: CompressionStreamResponse): Promise<void>;
    /**
     * Handles chunked page renders
     * @param firstFlushHandler
     * @param waitedFragments
     * @param chunkedFragmentReplacements
     * @param jsReplacements
     * @param isDebug
     * @param req
     * @param res
     */
    chunkedHandler(firstFlushHandler: Function, waitedFragments: IReplaceSet[], chunkedFragmentReplacements: IReplaceSet[], jsReplacements: IReplaceAsset[], isDebug: boolean, req: express.Request, res: CompressionStreamResponse): Promise<void>;
    /**
     * Creates a request handler from the compilation output. Express requests uses the return of this method
     * @param {Function} firstFlushHandler
     * @param {IReplaceSet[]} chunkedFragmentReplacements
     * @param {IReplaceSet[]} waitedFragments
     * @param {IReplaceAsset[]} jsReplacements
     * @param isDebug
     * @returns {(req: any, res: any) => void}
     */
    private buildHandler;
    /**
     * Flushes incoming fragment response
     * @param {IReplaceSet} chunkedReplacement
     * @param {IReplaceAsset[]} jsReplacements
     * @param res
     * @param isDebug
     * @returns {(fragmentContent: IFragmentContentResponse) => void}
     */
    private flush;
    /**
     * Clears html content from empty spaces
     * @param {string} str
     * @returns {string}
     */
    private static clearHtmlContent;
    /**
     * Replaces unfetched fragments with empty div error
     * @param {FragmentStorefront[]} fragments
     */
    private replaceUnfetchedFragments;
    /**
     * Creates chunked fragment containers
     * @param {FragmentStorefront[]} chunkedFragments
     * @returns {IReplaceSet[]}
     */
    private replaceChunkedFragmentContainers;
    /**
     * Creates containers for fragments should be waited
     * @param {FragmentStorefront[]} fragmentsShouldBeWaited
     * @param {IReplaceAsset[]} replaceJsAssets
     * @returns {IReplaceSet[]}
     */
    private replaceWaitedFragmentContainers;
    private static replaceCustomScripts;
}
export {};
